@@ -2,7 +2,7 @@
 
 #  ISC License
 #
-#  Copyright (c) 2020, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
+#  Copyright (c) 2020–2021, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
 #
 #  Permission to use, copy, modify, and/or distribute this software for any
 #  purpose with or without fee is hereby granted, provided that the above
@@ -66,7 +66,7 @@ class Debug:
         "Theme",
         "Usage_Dialog",
         "Version",
-        # "VispyCanvas",
+        "VispyCanvas",
         "Wire",
         "Wire_Presets",
         "Wire_Widget"
