@@ -2,7 +2,7 @@
 
 #  ISC License
 #
-#  Copyright (c) 2020, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
+#  Copyright (c) 2020–2021, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
 #
 #  Permission to use, copy, modify, and/or distribute this software for any
 #  purpose with or without fee is hereby granted, provided that the above
@@ -20,9 +20,12 @@ import os
 import time
 import atexit
 import datetime
+from typing import Optional
+
+import numpy as np
 import qtawesome as qta
 from PyQt5.QtCore import Qt, QThread, pyqtSignal, QLocale
-from PyQt5.QtWidgets import QMainWindow, QSplitter, QFileDialog, QDesktopWidget
+from PyQt5.QtWidgets import QMainWindow, QSplitter, QFileDialog, QDesktopWidget, QMessageBox
 from magneticalc.Assert_Dialog import Assert_Dialog
 from magneticalc.CalculationThread import CalculationThread
 from magneticalc.Config import Config
@@ -253,6 +256,60 @@ class GUI(QMainWindow):
         screen = QDesktopWidget().screenGeometry()
         self.setGeometry(0, 0, screen.width(), screen.height())
 
+    # ------------------------------------------------------------------------------------------------------------------
+
+    @staticmethod
+    def confirm_saving_unsaved_work(cancelable: bool) -> Optional[bool]:
+        """
+        Confirm saving unsaved work.
+
+        @param cancelable: True to make dialog cancelable, False to make dialog non-cancelable
+        @return: None if canceled, True if saving, False if discarding
+        """
+        messagebox = QMessageBox()
+        messagebox.setWindowTitle("Project Changed")
+        messagebox.setText("Do you want to save your changes?")
+        messagebox.setIcon(QMessageBox.Question)
+        messagebox.setStandardButtons(
+            (QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel) if cancelable else
+            (QMessageBox.Save | QMessageBox.Discard)
+        )
+        messagebox.setDefaultButton(QMessageBox.Save)
+        choice = messagebox.exec()
+        if choice == QMessageBox.Save:
+            return True
+        elif choice == QMessageBox.Discard:
+            return False
+        else:
+            return None
+
+    def confirm_close(self) -> None:
+        """
+        Called by menu "Quit" action.
+        Lets user choose to cancel closing or save / discard file if there is unsaved work.
+        """
+        if not self.config.get_synced():
+            choice = self.confirm_saving_unsaved_work(cancelable=True)
+            if choice is None:
+                Debug(self, ".confirm_close(): Canceled")
+                return
+            elif choice:
+                Debug(self, ".confirm_close(): Saving unsaved work")
+                self.config.save()
+            else:
+                Debug(self, ".confirm_close(): Discarding unsaved work")
+
+        self.close()
+
+    def closeEvent(self, _event):
+        """
+        Handles close event.
+
+        @param _event: Close event
+        """
+        Debug(self, ".closeEvent()")
+        self.quit()
+
     def quit(self):
         """
         Quits the application.
@@ -272,14 +329,6 @@ class GUI(QMainWindow):
         # Unregister exit handler (used by Assert_Dialog to exit gracefully)
         atexit.unregister(self.quit)
 
-    def closeEvent(self, _event):
-        """
-        Handles close event.
-
-        @param _event: Close event
-        """
-        self.quit()
-
     def keyPressEvent(self, event):
         """
         Handles key press event.
@@ -288,12 +337,12 @@ class GUI(QMainWindow):
         """
         if event.key() == Qt.Key_F2:
 
-            # Focus the the wire base points table
+            # Focus the  wire base points table
             self.sidebar_left.wire_widget.table.setFocus()
 
         elif event.key() == Qt.Key_F3:
 
-            # Open the the constraint editor
+            # Open the constraint editor
             self.sidebar_left.sampling_volume_widget.open_constraint_editor()
 
         elif event.key() == Qt.Key_F5:
@@ -355,6 +404,13 @@ class GUI(QMainWindow):
 
             self.model.invalidate()
 
+            if not self.config.get_synced():
+                if self.confirm_saving_unsaved_work(cancelable=False):
+                    Debug(self, ".file_open(): Saving unsaved work")
+                    self.config.save()
+                else:
+                    Debug(self, ".file_open(): Discarding unsaved work")
+
             self.config.close()
             self.config.set_filename(filename)
             self.config.load()
@@ -413,7 +469,6 @@ class GUI(QMainWindow):
         """
         Saves the currently displayed scene to PNG file.
         """
-
         filename, _chosen_extension = QFileDialog.getSaveFileName(
             parent=self,
             caption="Save Image",
@@ -429,3 +484,54 @@ class GUI(QMainWindow):
                 filename += ".png"
 
             self.vispy_canvas.save_image(filename)
+
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def import_wire(self):
+        """
+        Imports wire points from some TXT file.
+        """
+        filename, _chosen_extension = QFileDialog.getOpenFileName(
+            parent=self,
+            caption="Import Wire",
+            filter="Text File (*.txt)",
+            options=QFileDialog.DontUseNativeDialog
+        )
+
+        if filename != "":
+
+            points = np.loadtxt(filename)
+            self.sidebar_left.wire_widget.set_wire(
+                points=points,
+                stretch=[1.0, 1.0, 1.0],
+                rotational_symmetry={
+                    "count": 1,
+                    "radius": 0,
+                    "axis": 2,
+                    "offset": 0
+                }
+            )
+
+    def export_wire(self):
+        """
+        Exports wire points to some TXT file.
+        """
+        if not self.model.wire.is_valid():
+            Assert_Dialog(False, "Attempting to export invalid wire")
+            return
+
+        filename, _chosen_extension = QFileDialog.getSaveFileName(
+            parent=self,
+            caption="Export Wire",
+            directory=datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S_MagnetiCalc_Wire"),
+            filter="Text File (*.txt)",
+            options=QFileDialog.DontUseNativeDialog
+        )
+
+        if filename != "":
+            _file_name, file_extension = os.path.splitext(filename)
+
+            if file_extension.lower() != ".txt":
+                filename += ".txt"
+
+            np.savetxt(filename, self.model.wire.get_points_sliced())
